### Lambda函数配置

在编写和打包Lambda函数代码之后，除了选择触发函数的事件源之外，还需要设置各种配置项，以定义在Lambda中如何执行代码。

##### 函数内存

要定义分配给执行的Lambda函数的资源，你将获得一个单刻度表盘以增加/减少函数资源的：内存/RAM。可以为Lambda函数分配128MB到1.5GB的RAM。这不仅决定了函数代码在执行过程中可用的内存数量，而且同一个刻度还会影响函数可用的CPU和网络资源。

在优化任何Lambda函数的代价和性能时，选择适当的内存分配是非常重要的一步。请查看本白皮书后续的最佳实践，了解有关优化性能的更多细节。

##### 版本和别名

有时你可能需要引用或将Lambda函数恢复到以前部署的代码。Lambda可以让你**版本化**AWS Lambda函数。每个Lambda函数都有一个内置的默认版本: $LATEST。你可以通过 $LATEST 版本以使用最新上传到Lambda函数的代码。你可以对当前通过 $LATEST引用的代码执行快照，并通过[PublishVersion API](https://docs.aws.amazon.com/lambda/latest/dg/API_PublishVersion.html)创建版本编号。此外，通过[UpdateFunctionCode API](http://docs.aws.amazon.com/lambda/latest/dg/API_UpdateFunctionCode.html)更新函数代码时，还有一个可选的Boolean参数publish。通过在请求中设置publish：true，Lambda将创建一个新的Lambda函数版本，从上一个发布的版本开始递增。

你可以在任何时候独立地调用每个版本的Lambda函数。每个版本都有自己的 Amazon Resource Name(ARN)，如下所示：

    arn:aws:lambda:[region]:[account]:function:[fn_name]:[version]

在调用[Invoke API](https://docs.aws.amazon.com/lambda/latest/dg/API_Invoke.html)或为Lambda函数创建事件源时，还可以指定要执行的Lambda函数的特定版本。如果不提供版本号，或者使用不包含版本号的ARN，则默认情况下调用$LATEST。

知道Lambda函数容器特定于函数的某种版本这一点非常重要。因此，例如，如果在Lambda运行时环境中函数版本5的多个函数容器已经部署并可用，则相同函数的版本6将无法在现有版本5的容器之上执行 - 将为每个函数版本安装和管理一组不同的容器。

在测试和运维活动期间，通过版本号调用Lambda函数非常有用。但是，在实际应用流量中，我们不建议你通过指定版本的方式触发Lambda函数。这样做需要你在每次想要更新代码时更新所有调用Lambda函数的触发器和客户端，以指向一个新的函数版本。这里应该使用Lambda**别名**。函数别名允许调用事件源并将其指向特定的Lambda函数版本。

但是，你可以随时更新别名引用的版本。例如，通过别名live调用版本5的事件源和客户端可能会在将live别名更新为指向版本6时切换到函数的版本6。可以在ARN中引用每个别名，类似于引用函数版本号：

    arn:aws:lambda:[region]:[account]:function:[fn_name]:[alias]

以下是Lambda别名的一些示例建议以及如何使用它们：
- live/prod/active - 这可能代表生产触发器或客户端正在集成的Lambda函数版本。
- blue/green – 通过使用别名启用蓝/绿部署模式。
- debug - 如果你已经创建了一个测试堆栈来调试应用，那么当你需要执行更深入的分析时，它可以与这样的别名集成。

为函数别名的使用创建一个良好的、文档化的策略使你能够进行复杂的无服务器部署和操作实践。

##### IAM 角色

AWS身份和访问管理（IAM）提供了创建[IAM策略](http://docs.aws.amazon.com/IAM/latest/UserGuide/access_policies.html)的功能，这些策略定义了与AWS服务和API交互的权限。策略可以与**IAM角色**相关联。为特定角色生成的任何访问密钥ID和访问密钥都被授权执行附加到该角色的策略中所定义的操作。有关IAM最佳实践的更多信息，请参阅[本文档](https://docs.aws.amazon.com/IAM/latest/UserGuide/best-practices.html)。

在Lambda的上下文中，你为每个Lambda函数分配一个IAM角色（称为执行角色）。附加到该角色的IAM策略定义了函数代码被授权与哪些AWS服务API交互。有两个好处：
- 源代码不需要执行任何AWS凭据管理或轮换来与AWS API交互。只要使用AWS sdk和默认凭据提供程序，Lambda函数就会自动使用分配给该函数的执行角色相关联的临时凭据。
- 你的源代码与其自身的安全状态是解耦的。如果开发人员试图更改Lambda函数代码，以便与函数无法访问的服务集成，由于分配给函数的IAM角色，则该集成将失败。(除非他们使用了独立于执行角色的IAM凭证，否则您应该使用静态代码分析工具来确保您的源代码中不存在AWS凭证)。
  
为每个Lambda函数分配一个特定的、独立的和最少权限的IAM角色非常重要。这种策略确保每个Lambda函数都可以独立发展，而不需要增加任何其他Lambda函数的授权范围。

##### Lambda函数权限

你可以定义哪些推送模式事件源允许通过被称为**权限**的概念调用Lambda函数。使用权限，您可以声明**功能策略**，列出哪些AWS资源名称（ARN）允许调用函数。

对于拉取模型事件源（例如，Kinesis流和DynamoDB流），你需要确保分配给Lambda函数的IAM执行角色允许进行适当的操作。如果你不想管理所需的权限，AWS提供了一组与每个基于拉取事件源相关联的托管IAM角色。但是，为了确保最少权限IAM策略，你应该使用特定于资源的策略创建自己的IAM角色，以便仅允许访问预期的事件源。

##### 网络配置

通过使用AWS Lambda服务API的Invoke API执行Lambda函数；因此，没有直接的入站网络访问需要函数来管理。但是，函数代码可能需要与外部依赖项集成（内部或公共托管的Web服务，AWS服务，数据库等）。 Lambda函数有两个用于出站网络连接的广泛选项：
- 默认 - Lambda函数从Lambda管理的虚拟私有云(VPC)内部进行通信。它可以连接到internet，但不能连接到你自己的vpc中运行的任何私有部署的资源。
- VPC - Lambda函数通过弹性网络接口(ENI)进行通信，该接口在VPC中提供，你可以在自己的帐户中选择子网。可以为这些ENI分配安全组，并且流量将基于那些ENI所在的子网的路由表进行路由 - 就像将EC2实例放置在同一子网中一样。

如果Lambda函数不需要连接到任何私有部署的资源，我们建议选择默认网络选项。选择VPC选项需要你管理：
- 选择适当的子网以确保使用多个可用区以实现高可用性。
- 为每个子网分配适当数量的IP地址以容量管理。
- 实现VPC网络设计，允许Lambda函数具有所需的连接性和安全性。
- 如果Lambda函数调用模式需要及时创建新的ENI，则Lambda冷启动时间会增加。（当下创建ENI可能需要几秒钟。）

##### 环境变量

软件开发生命周期（SDLC）最佳实践指出开发人员分离他们的代码和配置。你可以通过将环境变量与Lambda一起使用来实现此目的。Lambda函数的环境变量使你能够动态地将数据传递到函数代码和库，而无需更改代码。环境变量是键-值对，你可以在函数配置中创建和修改它们。默认情况下，这些变量将在静止状态被加密。对于将存储为Lambda函数环境变量的任何敏感信息，我们建议在创建函数之前使用AWS Key Management Service（AWS KMS）加密这些值，并将加密的密文存储为变量值。然后让Lambda函数在执行时解密内存中的变量。

下面是一些你可能决定如何使用环境变量的示例：
- 日志设定（FATAL, ERROR, INFO, DEBUG, 等）
- 依赖关系和/或数据库连接字符串和凭据
- 功能标志和开关

Lambda函数的每个版本都可以拥有自己的环境变量值。然而，一旦为一个带编号的Lambda函数版本建立了值，它们就不能被更改。要更改Lambda函数环境变量，可以将它们更改为 $LATEST版本，然后发布包含新环境变量值的新版本。这使你可以始终跟踪哪些环境变量值与函数的先前版本相关联。这在回滚过程中或鉴别应用的过去状态时通常很重要。

##### 死信队列

即使在没有服务器的世界中，异常仍然可能发生。例如，(你可能已经上传了不允许成功解析Lambda事件的新函数代码，或者AWS中存在阻止调用该函数的操作事件)。对于异步事件源(InvocationType事件)，AWS拥有负责调用函数的客户端软件。AWS无法在调用发生时同步通知您调用是否成功。无论调用成功与否，AWS无法在调用发生时同步通知你。如果在这些模型中尝试调用函数时发生异常，则将再次尝试调用两次(在两次重试之间进行回退)。在第三次尝试之后，如果你为该函数配置了一个死信队列，那么该事件要么被丢弃，要么被放置到死信队列中。

SNS主题或SQS队列，是死信队列指定为所有失败调用事件的目的地。如果发生故障事件，使用死信队列只允许保留事件期间未能处理的消息。一旦函数能够再次被调用，就可以在死信队列中针对那些失败的事件进行重新处理。由你来决定重新处理/重试放置在死信队列中的函数调用尝试机制。有关死信队列的更多信息，[请参阅本教程](https://aws.amazon.com/cn/blogs/compute/robust-serverless-application-design-with-aws-lambda-dlq/)。如果对应用很重要的一点是，Lambda函数所有的调用最终都要完成，即使执行延迟，那么你应该使用死信队列。

#####  超时

你可以指定在返回超时之前允许单个函数执行完成的最大时间。Lambda函数在发布时的最大超时为300秒，这意味着对Lambda函数的一次调用不能执行超过300秒。你不应该总是将Lambda函数的超时设置为最大值。在许多情况下，应用程序应该快速失败。因为Lambda函数是根据执行时间以100毫秒为增量计费的，所以避免函数冗长超时可以防止函数只是在等待超时被计费（可能是外部依赖不可用，你不小心编写了一个无限循环，或其他类似的情况）。

此外，一旦执行完成或Lambda函数发生超时并返回响应，所有执行都将停止。这包括Lambda函数在执行期间可能生成的任何后台进程，子进程或异步进程。因此，你不应该依赖后台或异步流程来进行关键活动。代码应确保在超时之前完成这些活动或从函数返回响应。
