### Lambda函数事件源

现在你已经知道Lambda函数代码的组成部分，接下来让我们看看调用代码的**事件源**或触发器。虽然Lambda提供了调用API以能够直接调用函数，但你可能只会将其用于测试和运维。相反，你可以将Lambda函数与AWS服务中发生的事件源关联起来，这些事件源将根据需要调用函数。你不必编写、伸缩或维护将事件源与Lambda函数集成在一起的任何软件。

#### 调用模式
调用Lambda函数有两种模型：
- 推送模式 - 在另一个AWS服务中每次发生特定事件时，都会调用Lambda函数（比如，新对象被添加到S3 bucket）
- 拉取模式 - Lambda轮询数据源并使用到达数据源的任何新记录调用你的函数，在单个函数调用中将新记录批处理（比如，Amazon Kinesis或者Amazon DynamoDB流中的新纪录）

此外，Lambda函数可以同步或异步执行。触发Lambda函数时，你可以使用提供的参数**调用类型（InvocationType）**以选择执行类型，这个参数有三个可能的值：
- 请求响应 - 同步执行。
- 事件 - 同步执行。
- 排练（DryRun）- 测试调用方是否允许调用，但不要执行该函数。

每个事件源指定如何调用函数。事件源还负责生成自己的事件参数，如前所述。

下面的表提供了一些更流行的事件源如何与Lambda函数集成的详细信息。你可以在[这里](http://docs.aws.amazon.com/lambda/latest/dg/invoking-lambda-function.html)找到支持的事件源的完整列表。

#### 推送模式事件源

##### Amazon S3

|  |  |
| ------ | --------- |
|  调用模式  | 推送 |
|  调用类型  | 事件 |
|  说明  | S3事件通知(如ObjectCreated和ObjectRemoved)在发布时可以将其配置成调用Lambda函数|
|  用例案例  | 为用户通过应用程序上传到S3 bucket的图像创建图像修改（缩略图，不同分辨率，水印等）|
| | 处理上传到S3 bucket的原始数据，并将转换后的数据作为大数据管道的一部分移动到另一个S3 bucket|

##### Amazon API Gateway

|  |  |
| ------ | --------- |
|  调用模式  | 推送 |
|  调用类型  | 事件或请求响应 |
|  说明  | 使用API Gateway创建的API方法可以使用Lambda函数作为其服务后端。如果选择Lambda作为API方法的集成类型，则会同步调用Lambda函数（Lambda函数的响应用作API响应）。使用此集成类型，API Gateway还可以充当Lambda函数的简单代理。 API网关不会自行执行任何处理或转换，并将请求的所有内容传递给Lambda。|
| | 如果你希望API作为事件异步调用函数并立即返回空响应，则可以将API Gateway用作AWS服务代理(AWS Service Proxy)，并与Lambda调用API(Lambda Invoke API）集成，在请求标头中提供事件调用类型（Event InvocationType)。如果API客户端不需要基于请求返回任何信息并且希望获得尽可能快地响应时间，那么这是一个很好的选择。（此选项非常适合将用户在网站或应用上的交互推送到服务后端进行分析。）|
|  用例案例  | Web服务后端（Web应用，移动应用，微服务架构等）|
| | 遗留服务集成(Lambda函数，用于将遗留SOAP后端转换为新的现代REST API)。|
| | 任何其他用例，将HTTPS作为应用程序组件之间的恰当集成机制|

##### Amazon SNS

|  |  |
| ------ | --------- |
|  调用模式  | 推送 |
|  调用类型  | 事件 |
|  说明  | 发布到SNS主题的消息可以作为事件传递到Lambda函数。|
|  用例案例  | 自动响应CloudWatch告警。|
| | 处理来至于可以原生推送消息到SNS主题的其他服务（AWS或其他）的事件。|

##### AWS CloudFormation

|  |  |
| ------ | --------- |
|  调用模式  | 推送 |
|  调用类型  | 请求响应 |
|  说明  | 作为部署AWS CloudFormation堆栈的一部分，可以将Lambda函数指定为自定义资源，以执行任何自定义命令并将数据提供回正在进行的堆栈创建。|
|  用例案例  | 扩展AWS CloudFormation功能以囊括AWS CloudFormation本身尚未支持的AWS服务特性。|
| | 在堆栈创建/更新/删除过程的关键阶段执行自定义验证或报告。|

##### Amazon CloudWatch Events

|  |  |
| ------ | --------- |
|  调用模式  | 推送 |
|  调用类型  | 事件 |
|  说明  | 许多AWS服务将资源状态更改发布到CloudWatch Events。然后可以过滤这些事件并将其路由到Lambda函数以进行自动响应。|
|  用例案例  | 事件驱动的操作自动化（例如，每次启动新的EC2实例时执行操作，当AWS Trusted Advisor报告新的状态更改时通知适当的邮件列表）。|
| | 替换先前使用cron完成的任务（CloudWatch Events支持已预定事件）。|

##### Amazon Alexa

|  |  |
| ------ | --------- |
|  调用模式  | 推送 |
|  调用类型  | 请求响应 |
|  说明  | 你可以编写Lambda函数作为Amazon Alexa Skills的服务后端。当一个Alexa用户与你的技能互动，Alexa的自然语言理解和处理能力将传递他们的互动到你的Lambda函数。|
|  用例案例  | 你自己的Alexa技能。|

#### 拉取模式事件源

##### Amazon DynamoDB

|  |  |
| ------ | --------- |
|  调用模式  | 拉取 |
|  调用类型  | 请求/响应 |
|  说明  | Lambda将每秒多次轮询DynamoDB流，并使用自上次批处理以来已发布到流的更新批来调用Lambda函数。你可以配置每个调用的批处理大小。|
|  用例案例  | 以应用为中心的工作流，应该在DynamoDB表中发生更改时触发(例如，新用户注册、下单、接受好友请求等)。|
| | 将DynamoDB表复制到另一个区域(用于灾难恢复)或另一个服务(作为日志发送到S3 bucket进行备份或分析)。|

##### Amazon Kinesis Streams

|  |  |
| ------ | --------- |
|  调用模式  | 拉取 |
|  调用类型  | 请求/响应 |
|  说明  | Lambda轮询Kinesis流，每一个流分片一秒一次，并使用分片中的下一条记录调用Lambda函数。你可以为每次交付给函数的记录数量以及并发执行的Lambda函数容器的数量(流分片的数量=并发函数容器的数量)定义批大小。|
|  用例案例  | 大数据管道实时数据处理。|
| | 流日志语句或其它应用程序事件的实时告警/监控。|
