#### 可靠性最佳实践

可以构建无服务器应用来支持关键任务用例。就像任何任务关键型应用程序一样，重要的是，你要按照Werner Vogels（Amazon.com的CTO）所提倡的思维模式进行架构，“任何东西总是会失败”。对于无服务器应用程序，这可能意味着在代码中引入逻辑错误、应用程序依赖关系失败以及其他类似的应用程序级别的问题，应该尝试使用仍然适用于无服务器应用程序的现有最佳实践来预防和解决这些问题。对于基础设施级别的服务事件，即从事件中抽象出无服务器应用程序的事件，你应该了解如何对应用程序进行架构设计，以实现高可用性和容错性。

##### 高可用
高可用性对于生产应用程序非常重要。Lambda函数的可用性状态取决于可以在其中执行的可用区的数量。如果函数使用默认的网络环境，那么它可以在AWS区域内的所有可用区自动执行。在默认网络环境中，不需要为函数配置高可用性。如果函数部署在自己的VPC中，子网(及其各自的可用区)将定义在可用区停用时函数是否仍然可用。因此，VPC的设计包含多个可用区的子网是非常重要的。如果出现可用区宕机，剩余子网必须继续拥有足够的IP地址，以支持所需的并发函数数量，这一点非常重要。有关如何计算函数所需的IP地址数量的信息，请参阅[此文档](https://docs.aws.amazon.com/lambda/latest/dg/vpc.html#vpc-setup-guidelines)。

#### 容错
如果需要的应用可用性要求利用多个AWS区域，那么必须在设计前预先考虑这一点。将Lambda函数代码包复制到多个AWS区域并不复杂。与大多数多区域应用程序设计一样，复杂的是协调横跨应用堆栈中的所有层实现容错决定。这意味着你需要理解并编排到另一个AWS区域的转换，不仅要针对Lambda函数，还要针对事件源(以及事件源的上游依赖)和持久性层。最后，多区域架构是非常特定于应用。要使多区域设计可行，最重要的事情是在设计中预先考虑它。

#### 复原

思考下在函数无法被执行的情况下，无服务器应用的行为方式。对于使用API Gateway作为事件源的用例，这可以非常简单，只需优雅地处理错误消息，并提供可行的(如果降级)用户体验，直到函数能够再次被成功执行。对于异步用例，确保在停机期间没有函数调用丢失是非常重要的。为了确保在函数恢复后处理所有接收到的事件，你应该利用死信队列并实现在恢复后如何处理放置在该队列上的事件。
