#### 代码编写与代码管理最佳实践

在为Lambda函数开发代码时，有一些关于你应该如何编写和组织代码的具体建议，以便管理许多Lambda函数不会成为一项复杂的任务。

##### 代码编写最佳实践

根据使用的Lambda运行时语言，继续遵循为该语言建立的最佳实践。虽然围绕代码调用方式的环境已随Lambda发生了更改, 但语言运行时环境与其他任何地方相同。代码编写标准和最佳实践仍然适用。以下建议是特定于为Lambda编写代码，而不是所选择的语言的一般最佳实践。

###### 业务逻辑在Handler之外

Lambda函数在代码包中定义的处理函数处开始执行。在处理函数中，你应该会接收到Lambda提供的参数，将这些参数传递给另一个函数以解析应用上下文化的新变量/对象，然后转到位于处理函数和文件之外的业务逻辑。这使你能够创建尽可能与Lambda运行时环境解耦的代码包。这将极大地有利于在你所创建的对象和函数的上下文中测试代码的能力，并复用在Lambda之外的其他环境中编写的业务逻辑。

下面的示例(用Java编写)展示了应用程序的核心业务逻辑与Lambda紧密耦合的糟糕实践。在本例中，业务逻辑是在处理程序方法中创建的，并直接依赖于Lambda事件源对象。

![6](../../images/Figure6.jpg)

###### 容器预热 - 缓存/长连接/复用

如前所述，你应该编写利用预热函数容器的代码。这意味着以一种可能在后续调用中复用变量及其内容的方式确定变量的范围。这对于诸如引导配置、保持外部依赖关系连接的打开，或者可以从一个调用持续到下一个调用的大型对象的一次性初始化等情况尤其有效。

###### 依赖控制

Lambda执行环境包含许多库，例如用于Node.js的AWS SDK和Python运行时。（有关完整列表，请参阅[Lambda执行环境和可用库](https://docs.aws.amazon.com/lambda/latest/dg/current-supported-versions.html)。）为了启用最新的功能和安全更新，Lambda会定期更新这些库。这些更新可能会对Lambda函数的行为进行细微更改。要完全控制函数使用的依赖项，我们建议你使用部署包打包所有依赖项。

###### 依赖修剪

Lambda函数代码包在压缩时最多允许为50MB，在运行时环境中压缩时最多允许为250MB。如果在函数代码中包含大型依赖组件，则可能需要将所包含的依赖项修剪为仅运行时的必需项。这也允许在冷启动时更快地下载和安装Lambda函数代码。

###### 快速失败

为任何外部依赖项配置合理短的超时，以及合理短的总体Lambda函数超时。不要让函数在等待依赖项响应时无助地旋转。因为Lambda是根据函数执行的持续时间计费的，所以当函数依赖无响应时，你不希望产生高于必要的费用。

###### 异常处理

你可能决定根据Lambda的用例以不同的方式抛出和处理异常。如果你在Lambda函数的上游部署了API Gateway API，你可能决定将异常抛回API Gateway，API Gateway会基于发生的错误内容将其转换成合适的HTTP状态码与消息。如果你正在构建一个异步数据处理系统，那么你可能会认为代码库中的一些异常应该等同于将调用移动到死信队列以进行重新处理，而其他错误则可以直接记录下来，而不是放在死信队列中。你应该评估你所决定的失败行为是什么，并确保在代码中你正在创建和抛出正确类型的异常来实现此行为。要了解有关处理异常的更多信息，请参阅以下内容，了解如何为每种语言运行时环境定义异常：
- [Java](https://docs.aws.amazon.com/lambda/latest/dg/java-exceptions.html)
- [Node.js](https://docs.aws.amazon.com/lambda/latest/dg/nodejs-prog-mode-exceptions.html)
- [Python](https://docs.aws.amazon.com/lambda/latest/dg/python-exceptions.html)
- [C#](https://docs.aws.amazon.com/lambda/latest/dg/dotnet-exceptions.html)

##### 代码管理最佳实践

既然为Lambda函数编写的代码遵循了最佳实践，那么你应该如何管理这些代码呢？使用Lambda所支持的开发速度，你可能能够以典型流程所不熟悉的速度完成代码更改。无服务器架构所需的代码量减少了，这意味着Lambda函数代码代表了构成整个应用堆栈函数的很大一部分内容。因此，对Lambda函数代码进行良好的源代码管理将有助于确保安全、高效和平滑的变更管理流程。

###### 代码仓库组织

我们建议在选择源代码管理解决方案时，以高细粒度的方式组织Lambda函数源代码。这通常意味着Lambda函数和代码仓库或仓库项目（不同的源代码管理工具使用不同的词典。）之间有1:1的关系。但是，如果你遵循的策略是在相同逻辑的函数的不同生命周期阶段创建独立的Lambda函数(也就是说,你有两个Lambda函数,一个叫MyLambdaFunction-DEV和另一个为MyLambdaFunction-PROD)，那么这些独立的Lambda函数分享一个代码库是有意义的（部署可能是基于独立的发布分支）。

以这种方式组织代码的主要目的是帮助确保对特定Lambda函数的代码包作出贡献的所有代码都是独立版本控制和提交的，并定义其自己的依赖和那些依赖版本。每个Lambda函数都应该从源代码的角度与其他Lambda函数完全解耦，就像部署时一样。你不希望将应用架构现代化，使之模块化并与Lambda解耦，那么结果只会留下一个整体的、紧密耦合的代码。

###### 发布分支

我们建议创建仓库或项目分支策略，以便将Lambda函数部署与发布分支上的增量提交相关联。如果你无法自信地将仓库中的源代码更改与已部署到活动Lambda函数中的更改关联起来，那么操作调查总是从尝试确定当前部署的代码版本开始。你应该构建一个CI/CD管道（稍后提供更多建议），允许将Lambda代码包创建和部署时间与该Lambda函数的发布分支所发生的代码更改相关联。